/**
 * Copyright (c) 2001-2008 U-Mart Project
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 * ---------------------------------------------------------------------
 */
package strategy;

import java.util.*;

import cmdCore.*;
import serverCore.*;
import strategyCore.UBaseAgent;

/**
 * 헪NXłD
 */
public class Strategy extends UBaseAgent {

	/** 擾錻in̒ */
  public static final int NO_OF_SPOT_PRICES = 120;

  /** 擾敨in̒ */
  public static final int NO_OF_FUTURE_PRICES = 60;

  /**
   * RXgN^łD
   */
  public Strategy() {
    super("NotInitialized", "NotInitialized", "NotInitialized", 1);
  }

  /**
   * inԂ܂D
   * @return in
   */
  private int[] getSpotPrices() {
    UCSpotPriceCore cmd = (UCSpotPriceCore) fUmcp.getCommand("SpotPrice");
    cmd.setArguments("j30", Strategy.NO_OF_SPOT_PRICES);
    UCommandStatus status = cmd.doIt();
    if (status.getStatus() == false) {
      System.err.println("UserName:" + fLoginName + ", UserID:" + fUserID);
      System.err.println(status.getErrorMessage() +
                         " in Strategy.getSpotPrices");
      System.exit(5);
    }
    ArrayList spotList = cmd.getResults();
    if (spotList.size() != Strategy.NO_OF_SPOT_PRICES) {
      System.err.println(
          "spotList.size() != this.NO_OF_SPOT_PRICES in Strategy.doActions");
      System.exit(5);
    }
    int[] spotPrices = new int[Strategy.NO_OF_SPOT_PRICES];
    for (int i = 0; i < Strategy.NO_OF_SPOT_PRICES; ++i) {
      HashMap elem = (HashMap) spotList.get(i);
      spotPrices[Strategy.NO_OF_SPOT_PRICES - i -
          1] = (int) ( (Long) elem.get(UCSpotPriceCore.LONG_PRICE)).longValue();
    }
    return spotPrices;
  }

  /**
   * 敨inԂ܂D
   * @return 敨in
   */
  private int[] getFuturePrices() {
    UCFuturePriceCore cmd =
        (UCFuturePriceCore) fUmcp.getCommand("FuturePrice");
    cmd.setArguments("j30", Strategy.NO_OF_FUTURE_PRICES);
    UCommandStatus status = cmd.doIt();
    if (status.getStatus() == false) {
      System.err.println("UserName:" + fLoginName + ", UserID:" + fUserID);
      System.err.println(status.getErrorMessage() +
                         " in Strategy.getFuturePrices");
      System.exit(5);
    }
    ArrayList futureList = cmd.getResults();
    if (futureList.size() != Strategy.NO_OF_FUTURE_PRICES) {
      System.err.println("futureList.size():" + futureList.size() +
                         " != this.NO_OF_FUTURE_PRICES" + NO_OF_FUTURE_PRICES +
                         " in Strategy.getFuturePrices");
      System.exit(5);
    }
    int[] futurePrices = new int[Strategy.NO_OF_FUTURE_PRICES];
    for (int i = 0; i < Strategy.NO_OF_FUTURE_PRICES; ++i) {
      HashMap elem = (HashMap) futureList.get(i);
      futurePrices[Strategy.NO_OF_FUTURE_PRICES - i -
          1] = (int) ( (Long) elem.get(UCFuturePriceCore.LONG_PRICE)).longValue();
    }
    return futurePrices;
  }

  /**
   * |WVԂ܂D
   * @return |WV
   */
  private int getPosition() {
    UCPositionCore cmd = (UCPositionCore) fUmcp.getCommand("Position");
    UCommandStatus status = cmd.doIt();
    if (status.getStatus() == false) {
      System.err.println("UserName:" + fLoginName + ", UserID:" + fUserID +
                         " in Strategy.getPosition");
      System.err.println(status.getErrorMessage() + " in Strategy.getPosition");
      System.exit(5);
    }
    HashMap result = cmd.getResults();
    long todayBuy = ( (Long) result.get(UCPositionCore.LONG_TODAY_BUY)).
        longValue();
    long todaySell = ( (Long) result.get(UCPositionCore.LONG_TODAY_SELL)).
        longValue();
    long yesterdayBuy = ( (Long) result.get(UCPositionCore.LONG_YESTERDAY_BUY)).
        longValue();
    long yesterdaySell = ( (Long) result.get(UCPositionCore.LONG_YESTERDAY_SELL)).
        longValue();
    long buy = todayBuy + yesterdayBuy;
    long sell = todaySell + yesterdaySell;
    return (int) (buy - sell);
  }

  /**
   * cԂ܂D
   * @return c
   */
  private long getMoney() {
    UCBalancesCore cmd = (UCBalancesCore) fUmcp.getCommand("Balances");
    UCommandStatus status = cmd.doIt();
    if (status.getStatus() == false) {
      System.err.println("UserName:" + fLoginName + ", UserID:" + fUserID);
      System.err.println(status.getErrorMessage() + " in Strategy.getMoney");
      System.exit(5);
    }
    HashMap bal = cmd.getTodayResults();
    return ( (Long) bal.get(UCBalancesCore.LONG_SURPLUS)).longValue();
  }

  /**
   * Z܂ł̐ߐԂ܂D
   * @param date@
   * @param boardNo ߐ
   * @param maxDate 
   * @param noOfBoardsPerDay P̐ߐ
   * @return
   */
  private int getRestDay(int date, int boardNo,
                         int maxDate, int noOfBoardsPerDay) {
    return (maxDate - date) * noOfBoardsPerDay + noOfBoardsPerDay - boardNo + 1;
  }

  /**
   * ߂̉iԂ܂D
   * @param prices in
   * @return ߂̉i
   */
  public int getLatestPrice(int[] prices) {
    for (int i = prices.length - 1; i >= 0; --i) {
      if (prices[i] >= 0) {
        return prices[i];
      }
    }
    return -1;
  }

  /**
   * @see strategyCore.UBaseAgent#doActions(int, int, int, int, int)
   */
  public void doActions(int date, int session, int serverState,
                        int maxDate, int noOfSessionsPerDay) {
    if (serverState != UServerStatus.ACCEPT_ORDERS) {
      return;
    }
    int[] spotPrices = getSpotPrices();
    int[] futurePrices = getFuturePrices();
    int pos = getPosition();
    long money = getMoney();
    int restDay = getRestDay(date, session, maxDate, noOfSessionsPerDay);
    action(spotPrices, futurePrices, pos, money, restDay);
  }

  /**
   * P̒肷郁\bhłDĎ헪܂D̃\bh́Caction\bhȂꍇC1߂1R[܂D
   * @param spotPrices ǐnDzvf0Ԗڂ119Ԗڂ܂ł120i[ĂD119ԖڂŐV̌iD
   * @param futurePrices 敨ǐnDzvf0Ԗڂ59Ԗڂ܂ł60i[ĂD59ԖڂŐV̉iDsꂪJO͌iςɑĂD܂CsŎɉi肵Ȃꍇ͒l'-1'ĂD
   * @param position G[WFg̃|WVDȂΔzCȂΔz\D
   * @param money G[WFgۗ̕L錻cD^longł邱Ƃɒӂ邱ƁD
   * @param restSessions sꂪ܂ł̎cĂߐi񂹂̉񐔁j
   * @return 
   */
  public Order getOrder(int[] spotPrices, int[] futurePrices, int position,
                        long money, int restSessions) {
    return null;
  }

  /**
   * 邽߂̃\bhłD
   * PłP̒̂Ȃ΁Ĉ܂܂ɂĂ܂D
   * ̒ŝȂ΁CgetOrderȊÕ\bhpӂCorderRequest\bh𕡐Ăׂ΂悢D
   * @param spotPrices ǐnDzvf0Ԗڂ119Ԗڂ܂ł120i[ĂD119ԖڂŐV̌iD
   * @param futurePrices 敨ǐnDzvf0Ԗڂ59Ԗڂ܂ł60i[ĂD59ԖڂŐV̉iDsꂪJO͌iςɑĂD܂CsŎɉi肵Ȃꍇ͒l'-1'ĂD
   * @param position G[WFg̃|WVDȂΔzCȂΔz\D
   * @param money G[WFgۗ̕L錻cD^longł邱Ƃɒӂ邱ƁD
   * @param restSessions sꂪ܂ł̎cĂߐi񂹂̉񐔁j
   */
  public void action(int[] spotPrices, int[] futurePrices, int position, long money, int restSessions) {
    Order o = getOrder(spotPrices, futurePrices, position, money, restSessions);
    if (o == null) {
      System.err.println("Strategy.getOrder returned null in Strategy.action");
      return;
    }
    orderRequest(o);
  }

  /**
   * T[o֒𑗂܂D
   * @param o 
   */
  public void orderRequest(Order o) {
    UCOrderRequestCore cmd = (UCOrderRequestCore) fUmcp.getCommand("OrderRequest");
    int sellBuy;
    if (o.buysell == Order.BUY) {
      sellBuy = UOrder.BUY;
    } else if (o.buysell == Order.SELL) {
      sellBuy = UOrder.SELL;
    } else {
      return;
    }
    if (o.price <= 0 || o.quant <= 0) {
      // System.err.println("ERORR!!!");
      // System.err.println("price=" + o.price + ", quant=" + o.quant);
      try {
        throw new Exception();
      } catch (Exception e) {
        // TODO Auto-generated catch block
        // e.printStackTrace();
        return;
      }
    }
    cmd.setArguments("j30", UOrder.NEW, sellBuy, UOrder.LIMIT, o.price, o.quant);
    UCommandStatus status = cmd.doIt();
    if (status.getStatus() == false) {
      System.err.println("UserName:" + fLoginName + ", UserID:" + fUserID);
      System.err.println(status.getErrorMessage() + " in Strategy.orderRequest");
    }
  }

}
